/*
 *   Copyright  2012, Christian Rosentreter. All Rights Reserved.
 *
 *   Basing on deutsch.language, which is
 *   Copyright  1995-2001, AROS. All Rights Reserved.
 *   Copyright  2001-2009, MorphOS Team. All Rights Reserved.
 *
 *   $Id: esperanto.c,v 1.1 2012/01/08 01:18:11 tokai Exp $
 *
 *   Desc: esperanto.language description file.
 *   Lang: esperanto
 */

#include <exec/types.h>
#include <aros/system.h>
#include <exec/resident.h>
#include <exec/libraries.h>
#include <libraries/locale.h>
#include <dos/dos.h>

#include <proto/exec.h>
#include <aros/libcall.h>

#include <aros/debug.h>

#define LANGSTR "esperanto"   /* String version of above     */
#define LANGVER 51            /* Version number of language  */
#define LANGREV 1             /* Revision number of language */

#define LANGTAG "\0$VER: esperanto.language 51.1 (08.01.2012)  2012 Christian Rosentreter"



/* ----------------------------------------------------------------------- */

/*  Bit masks for locale .language functions. Only implement GetString() 
 */
#define LF_GetLangStr       (1L << 3)
#define LF_GetLangStrU      (1L << 17)

#define LF_MASK             (LF_GetLangStr | LF_GetLangStrU)


/*  Arrays for Esperanto character type/conversion 
 */
extern const STRPTR        __esperanto_strings[];
extern const CONST_WSTRPTR __esperanto_stringsu[];




/*  -------------------------------------------------------------------------
 *  Library definition, you should not need to change any of this.
 *  -------------------------------------------------------------------------
 */

struct Language
{
    struct Library library;
    struct ExecBase *sysbase;
    BPTR seglist;
};

extern const UBYTE name[];
extern const UBYTE version[];
extern const APTR inittabl[4];
extern void *const functable[];
extern struct Language *AROS_SLIB_ENTRY(init,language)();
extern struct Language *AROS_SLIB_ENTRY(open,language)();
extern BPTR AROS_SLIB_ENTRY(close,language)();
extern BPTR AROS_SLIB_ENTRY(expunge,language)();
extern int AROS_SLIB_ENTRY(null,language)();
extern ULONG AROS_SLIB_ENTRY(mask,language)();
extern const char end;

int entry(void)
{
	return -1;
}

const struct Resident languageTag =
{
	RTC_MATCHWORD,
	(struct Resident *)&languageTag,
	(APTR)&languageTag + 1,
	RTF_PPC | RTF_AUTOINIT,
	LANGVER,
	NT_LIBRARY,
	-120,
	(STRPTR)name,
	(STRPTR)&version[7],
	(ULONG *)inittabl
};

const UBYTE name[]    = LANGSTR ".language";
const UBYTE version[] = LANGTAG;

const APTR inittabl[4] =
{
	(APTR)sizeof(struct Language),
	(APTR)functable,
	(APTR)NULL,
	&AROS_SLIB_ENTRY(init,language)
};

struct ExecBase *mySysBase;

#ifdef __MORPHOS__
struct Language *LIB_init(struct Language *language, BPTR segList, struct ExecBase *SysBase)
#else
AROS_LH2(struct Language *, init,
    AROS_LHA(struct Language *, language, D0),
    AROS_LHA(BPTR,             segList, A0),
    struct ExecBase *, SysBase, 0, language)
#endif
{
	AROS_LIBFUNC_INIT

	/*  You could just as easily do this bit as the InitResident()
	 *  datatable, however this works just as well.
	 */
	language->library.lib_Node.ln_Type                = NT_LIBRARY;
	language->library.lib_Node.ln_Pri                 = -120;
	(const UBYTE *)language->library.lib_Node.ln_Name = name;
	language->library.lib_Flags                       = LIBF_SUMUSED | LIBF_CHANGED;
	language->library.lib_Version                     = LANGVER;
	language->library.lib_Revision                    = LANGREV;
	(const UBYTE *)language->library.lib_IdString     = &version[7];

	language->seglist = segList;
	language->sysbase = SysBase;
	mySysBase         = SysBase;

	/*  Although it is unlikely, you would return NULL if you for some
	 *  unknown reason couldn't open.
	 */	
	return language;
}

#define SysBase language->sysbase

AROS_LH1(struct Language *, open,
	AROS_LHA(ULONG, version, D0), 
	struct Language *, language, 1, language)
{
	AROS_LIBFUNC_INIT

	language->library.lib_OpenCnt++;
	language->library.lib_Flags &= ~LIBF_DELEXP;

	/* Again return NULL if you could not open 
	 */
	return language;

	AROS_LIBFUNC_EXIT
}

AROS_LH0(BPTR, close, struct Language *, language, 2, language)
{
	AROS_LIBFUNC_INIT

	if(! --language->library.lib_OpenCnt)
	{
		/*  Delayed expunge pending? 
		 */
		if(language->library.lib_Flags & LIBF_DELEXP)
		{
			/* Yes, expunge the library
			 */
			return AROS_LC0(BPTR, expunge, struct Language *, language, 3, language);
		}
	}

	return NULL;

	AROS_LIBFUNC_EXIT
}


AROS_LH0(BPTR, expunge, struct Language *, language, 3, language)
{
	AROS_LIBFUNC_INIT

	BPTR ret;

	if(language->library.lib_OpenCnt)
	{
		/* Can't expunge, we are still open
		 */
		language->library.lib_Flags |= LIBF_DELEXP;
		return 0;
	}

	REMOVE(&language->library.lib_Node);
	ret = language->seglist;

	FreeMem((UBYTE *)language - language->library.lib_NegSize,
		language->library.lib_PosSize + language->library.lib_NegSize);

	return ret;

	AROS_LIBFUNC_EXIT
}


AROS_LH0I(int, null, struct Language *, language, 4, language)
{
	AROS_LIBFUNC_INIT

	return 0;

	AROS_LIBFUNC_EXIT
}




/*  ------------------------------------------------------------------------
 *  Language specific functions
 *  ------------------------------------------------------------------------
 */

#undef SysBase
#define SysBase	mySysBase

/*  ULONG LanguageMask():
 *
 *  This function is to inform locale.library what functions it should
 *  use from this library. This is done by returning a bitmask containing
 *  1's for functions to use, and 0's for functions to ignore.
 *
 *  Unused bits MUST be 0 for future compatibility.
 */
AROS_LH0(ULONG, mask, struct Language *, language, 5, language)
{
	AROS_LIBFUNC_INIT

	return ( LF_MASK );

	AROS_LIBFUNC_EXIT
}

/*  STRPTR GetLangString(ULONG num): Language function 3
 *
 *  This function is called by GetLocaleStr() and should return
 *  the string matching the string id passed in as num.
 */
AROS_LH1(STRPTR, getlangstring,
    AROS_LHA(ULONG, id, D0),
    struct LocaleBase *, LocaleBase, 9, language)
{
	AROS_LIBFUNC_INIT

	if(id < MAXSTRMSG)
	{
		return __esperanto_strings[id];
	}
	else
	{
		return NULL;
	}

	AROS_LIBFUNC_EXIT
}

CONST_WSTRPTR LIB_getlangstringu(ULONG id, struct LocaleBase *LocaleBase)
{
	if (id < MAXSTRMSG)
	{
		return __esperanto_stringsu[id];
	}
	else
	{
		return NULL;
	}
}


/*  -----------------------------------------------------------------------
 *  Library function table - you will need to alter this
 *  I have this right here at the end of the library so that I do not
 *  have to have prototypes for the functions. Although you could do that.
 *  -----------------------------------------------------------------------
 */
void *const functable[] =
{
	(void *const) FUNCARRAY_BEGIN,
	(void *)FUNCARRAY_32BIT_NATIVE,
	&AROS_SLIB_ENTRY(open,language),
	&AROS_SLIB_ENTRY(close,language),
	&AROS_SLIB_ENTRY(expunge,language),
	&AROS_SLIB_ENTRY(null,language),
	&AROS_SLIB_ENTRY(mask,language),

	/* Note, shorter function table, as only getlangstring is used
	 */

	/* 0 - 3 */
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),    
	&AROS_SLIB_ENTRY(getlangstring, language),

	/* 4 - 16 */
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	(void *)-1,

	/* 17 */
	(void *) FUNCARRAY_32BIT_SYSTEMV,
	&LIB_getlangstringu,
	(void *)-1L,

	(void *const) FUNCARRAY_END
};


/* Note: Strings are encoded with ISO-8859-3 -- tokai */


/*  Note how only the required data structures are kept...
 *
 *  This is the list of strings. It is an array of pointers to strings,
 *  although how it is laid out is implementation dependent.
 */
static const STRPTR __esperanto_strings[] =
{
	/* A blank string
	 */
	"",

	/*  nomo de tago de la semanjno
	 */
	"diman" "\xE6" "o",     /* includes a c-circumflex, U+0109 */
	"lundo",
	"mardo",
	"merkredo", 
	"\xBC" "a" "\xFD" "do", /* includes j-circumflex, and u-breve */
	"vendredo",
	"sabato",

	/*  mallongigo de nomo de tago de la semanjno
	 */
	"di", 
	"lu",
	"ma",
	"me", 
	"\xBC" "a", /* includes j-circumflex */
	"ve",
	"sa",

    /*  nomo de monato 
	 */
	"januaro",  "februaro",         "marto",
	"aprilo",   "majo",             "junio",
	"julio",    "a" "\xFD" "gusto", "septembro",
	"oktobro",  "novembro",         "decembro",

	/*  mallongigo de nomo de monato
	 *
	 *  (XXX: Maybe these should be all in capitals, like SEP, JAN, etc. (e.g. small 'sep' can also mean "seven")
	 */
	"jan",  "feb",          "mar",
	"apr",  "maj",          "jun",
	"jul",  "a" "\xFD" "g", "sep",
	"okt",  "nov",          "dec",

	/*  Responses
	 */
	"jes",   /* Yes, affirmative response */
	"ne",    /* No,  negative response    */

	/*  AM/PM strings
	 *
	 *  AM 0000 -> 1159,
	 *  PM 1200 -> 2359
	 */
	"a.t.m." /* antautagmeze */, "p.t.m." /* posttagmeze */,

	/*  Soft and hard hyphens
	 */
	"\xAD", "-",

	/*  Open and close quotes
	 */
	"\"", "\"", /* should use << >> kind of quotes, but they are not available in Latin3, but we'll use them for unicode variants at least */

	/*  Days: But not actual day names
	 *
	 *  Yesterday - the day before the current
	 *  Today     - the current day
	 *  Tomorrow  - the next day
	 *  Future.
	 */
	"hiera" "\xFD",
	"hodia" "\xFD",
	"morga" "\xFD",
	"estonteco"
};

static const ULONG esperanto_sunday[]         = {'d','i','m','a','n', 0x0109, 'o', '\0'};
static const ULONG esperanto_thursday[]       = {0x0135, 'a', 0x016D, 'd', 'o', '\0'};
static const ULONG esperanto_thursday_short[] = {0x0135, 'a', '\0'};
static const ULONG esperanto_softhyp[]        = {0x00AD, '\0'};
static const ULONG esperanto_august[]         = {'a', 0x016D, 'g', 'u', 's', 't', 'o', '\0'};
static const ULONG esperanto_august_short[]   = {'a', 0x016D, 'g', '\0'};
static const ULONG esperanto_openquote[]      = {0x00AB, '\0'}; /* Left-pointing double angle quotation mark  (guillemet) */
static const ULONG esperanto_closequote[]     = {0x00BB, '\0'}; /* Right-pointing double angle quotation mark             */
static const ULONG esperanto_yesterday[]      = {'h', 'i', 'e', 'r', 'a', 0x016D, '\0'};
static const ULONG esperanto_today[]          = {'h', 'o', 'd', 'i', 'a', 0x016D, '\0'};
static const ULONG esperanto_tomorrow[]       = {'m', 'o', 'r', 'g', 'a', 0x016D, '\0'};

static const CONST_WSTRPTR __esperanto_stringsu[] =
{
	/* A blank string
	 */
	(CONST_WSTRPTR)L"",

	/*  The days of the week. Starts with the first day of the week.
	 *
	 *  In English this would be Sunday, this depends upon the settings
	 *  of Locale->CalendarType.
	 *
	 *  #warning: stegerg: I think this must always start with Sunday and not what comment above says
	 */
	(CONST_WSTRPTR)esperanto_sunday,
	(CONST_WSTRPTR)L"lundo",
	(CONST_WSTRPTR)L"mardo",
	(CONST_WSTRPTR)L"merkredo",
	(CONST_WSTRPTR)esperanto_thursday,
	(CONST_WSTRPTR)L"vendredo",
	(CONST_WSTRPTR)L"sabato",

	/*  Abbreviated days of the week
	 */
	(CONST_WSTRPTR)L"di",
	(CONST_WSTRPTR)L"lu",
	(CONST_WSTRPTR)L"ma",
	(CONST_WSTRPTR)L"me",
	(CONST_WSTRPTR)esperanto_thursday_short,
	(CONST_WSTRPTR)L"ve",
	(CONST_WSTRPTR)L"sa",
	
    /*  Months of the year 
	 */
	(CONST_WSTRPTR)L"januaro",  (CONST_WSTRPTR)L"februaro",      (CONST_WSTRPTR)L"marto",
	(CONST_WSTRPTR)L"aprilo",   (CONST_WSTRPTR)L"majo",          (CONST_WSTRPTR)L"junio",
	(CONST_WSTRPTR)L"julio",    (CONST_WSTRPTR)esperanto_august, (CONST_WSTRPTR)L"septembro",
	(CONST_WSTRPTR)L"oktobro",  (CONST_WSTRPTR)L"novembro",      (CONST_WSTRPTR)L"decembro",

	/*  Abbreviated months of the year
	 */
	(CONST_WSTRPTR)L"jan", (CONST_WSTRPTR)L"feb",                 (CONST_WSTRPTR)L"mar",
	(CONST_WSTRPTR)L"apr", (CONST_WSTRPTR)L"maj",                 (CONST_WSTRPTR)L"jun",
	(CONST_WSTRPTR)L"jul", (CONST_WSTRPTR)esperanto_august_short, (CONST_WSTRPTR)L"sep",
	(CONST_WSTRPTR)L"okt", (CONST_WSTRPTR)L"nov",                 (CONST_WSTRPTR)L"dec",

	/*  Responses
	 */
	(CONST_WSTRPTR)L"jes",   /* Yes, affirmative response */
	(CONST_WSTRPTR)L"ne",    /* No,  negative response    */

	/*  AM/PM strings
	 *
	 *  AM 0000 -> 1159,
	 *  PM 1200 -> 2359
	 */
	(CONST_WSTRPTR)L"a.t.m.", (CONST_WSTRPTR)L"p.t.m.",

	/*  Soft and hard hyphens
	 */
	(CONST_WSTRPTR)esperanto_softhyp, (CONST_WSTRPTR)L"-",

	/*  Open and close quotes
	 */
	(CONST_WSTRPTR)esperanto_openquote, (CONST_WSTRPTR)esperanto_closequote,

	/*  Days: But not actual day names
	 *
	 *  Yesterday - the day before the current
	 *  Today     - the current day
	 *  Tomorrow  - the next day
	 *  Future.
	 */
	(CONST_WSTRPTR)esperanto_yesterday,
	(CONST_WSTRPTR)esperanto_today,
	(CONST_WSTRPTR)esperanto_tomorrow,
	(CONST_WSTRPTR)L"estonteco"
};
